==> pkg_simplecustomrouter-0.1.zip: unit-tests/plgSystemSimpleCustomRouterTest.php <==
<?php
/**
 * @copyright	Copyright (C) 2012 Daniel Calviño Sánchez
 * @license		GNU Affero General Public License version 3 or later; see LICENSE.txt
 */

require_once JPATH_BASE.'/tests/unit/JoomlaTestCase.php';

require_once JPATH_BASE.'/plugins/system/simplecustomrouter/simplecustomrouter.php';
require_once JPATH_BASE.'/libraries/joomla/event/dispatcher.php';

//Needed by JApplication
require_once JPATH_BASE.'/libraries/joomla/environment/request.php';
require_once JPATH_BASE.'/libraries/joomla/environment/uri.php';

class plgSystemSimpleCustomRouterTest extends JoomlaTestCase {

    protected function setUp() {
        JFactory::$config = null;
        
        //Set a JSession stub, as creating a real JSession fails (see the
        //comments on JSessionStub class)
        JFactory::$session = new JSessionStub();

        JFactory::$application = null;
        //It is always the same JApplication, as the instances are stored in a
        //static variable internal to a method.
        JFactory::getApplication('site', array('session' => false));
        //The same JApplication always has the same router, so the router rules
        //have to be emptied between tests.
        $this->setPrivate(JFactory::getApplication()->getRouter(), '_rules', array());
    }
    
    public function testAttachRulesOnInitialization() {
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();

        //Check that the SimpleCustomRouter was added to the rules of the router
        $router = JFactory::getApplication()->getRouter();
        $routerRules = $this->getPrivate($router, '_rules');
        
        $this->assertEquals(1, count($routerRules['build']), 'Build rules count');
        $this->assertEquals('SimpleCustomRouter', get_class($routerRules['build'][0][0]), 'Build rule class');
        $this->assertEquals('build', $routerRules['build'][0][1], 'Build rule method');

        $this->assertEquals(1, count($routerRules['parse']), 'Parse rules count');
        $this->assertEquals('SimpleCustomRouter', get_class($routerRules['parse'][0][0]), 'Parse rule class');
        $this->assertEquals('parse', $routerRules['parse'][0][1], 'Parse rule method');
        
        $this->assertTrue($routerRules['build'][0][0] == $routerRules['parse'][0][0], 'Same object in build and parse rules');
    }

    public function testAttachRulesOnAdminInitialization() {
        //Needed for JURI, which is needed by JAdministrator 
        $_SERVER['HTTP_HOST'] = 'localhost';
        
        JFactory::$application = null;
        JFactory::getApplication('administrator', array('session' => false));
        $this->setPrivate(JFactory::getApplication()->getRouter(), '_rules', array());

        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();

        //Check that the SimpleCustomRouter was not added to the rules of the
        //router
        $router = JFactory::getApplication()->getRouter();
        $routerRules = $this->getPrivate($router, '_rules');
        
        //As the rules were emptied there will be no 'build' or 'parse' elements
        $this->assertEquals(0, count($routerRules), 'Rules count');
    }
    
    private function getPrivate($object, $property) {
        $reflectionProperty = new ReflectionProperty(get_class($object), $property);
        $reflectionProperty->setAccessible(true);
        return $reflectionProperty->getValue($object);
    }

    private function setPrivate($object, $property, $value) {
        $reflectionProperty = new ReflectionProperty(get_class($object), $property);
        $reflectionProperty->setAccessible(true);
        $reflectionProperty->setValue($object, $value);
    }
    
    public function testCreateCacheDefaultEnabled() {
        $this->setDefaultCacheConfiguration(true, 'file', '15');
                
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNotNull($simpleCustomRouterCache, 'Cache set');
        $this->assertEquals(JFactory::getCache('simplecustomrouter', ''), $simpleCustomRouterCache, 'Cache same as "simplecustomrouter" cache');
        $this->assertTrue($simpleCustomRouterCache->getCaching(), 'Cache enabled');
        $this->assertEquals('JCacheStorageFile', get_class($simpleCustomRouterCache->_getStorage()), 'Cache storage'); 
        $this->assertEquals('15', $simpleCustomRouterCache->cache->_options['lifetime'], 'Cache lifetime'); 
    }

    public function testCreateCacheDefaultEnabledWithoutHandler() {
        $this->setDefaultCacheConfiguration(true, '', '15');
                
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    public function testCreateCacheDefaultDisabled() {
        $this->setDefaultCacheConfiguration(false, 'file', '15');
                
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    public function testCreateCacheDefaultEnabledWhenCustomDisabled() {
        $this->setDefaultCacheConfiguration(true, 'file', '15');
                
        $params = '{"overrideDefaultConfiguration":false, "enableCache":false}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNotNull($simpleCustomRouterCache, 'Cache set');
        $this->assertEquals(JFactory::getCache('simplecustomrouter', ''), $simpleCustomRouterCache, 'Cache same as "simplecustomrouter" cache');
        $this->assertTrue($simpleCustomRouterCache->getCaching(), 'Cache enabled');
        $this->assertEquals('JCacheStorageFile', get_class($simpleCustomRouterCache->_getStorage()), 'Cache storage'); 
        $this->assertEquals('15', $simpleCustomRouterCache->cache->_options['lifetime'], 'Cache lifetime'); 
    }

    public function testCreateCacheDefaultEnabledWhenCustomDisabledWithoutHandler() {
        $this->setDefaultCacheConfiguration(true, '', '15');
                
        $params = '{"overrideDefaultConfiguration":false, "enableCache":false}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }
    
    public function testCreateCacheDefaultDisabledWhenCustomEnabled() {
        $this->setDefaultCacheConfiguration(false, 'file', '15');
                
        $params = '{"overrideDefaultConfiguration":false, "enableCache":true}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }
    
    public function testCreateCacheCustomEnabledWhenDefaultDisabled() {
        $this->setDefaultCacheConfiguration(false, 'file', '15');
        
        $params = '{"overrideDefaultConfiguration":true, "enableCache":true}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNotNull($simpleCustomRouterCache, 'Cache set');
        $this->assertEquals(JFactory::getCache('simplecustomrouter', ''), $simpleCustomRouterCache, 'Cache same as "simplecustomrouter" cache');
        $this->assertTrue($simpleCustomRouterCache->getCaching(), 'Cache enabled');
        $this->assertEquals('JCacheStorageFile', get_class($simpleCustomRouterCache->_getStorage()), 'Cache storage'); 
        $this->assertEquals('15', $simpleCustomRouterCache->cache->_options['lifetime'], 'Cache lifetime'); 
    }

    public function testCreateCacheCustomEnabledWhenDefaultDisabledWithoutHandler() {
        $this->setDefaultCacheConfiguration(false, '', '15');
                
        $params = '{"overrideDefaultConfiguration":true, "enableCache":true}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    public function testCreateCacheCustomDisabledWhenDefaultEnabled() {
        $this->setDefaultCacheConfiguration(true, 'file', '15');
                
        $params = '{"overrideDefaultConfiguration":true, "enableCache":false}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    private function setDefaultCacheConfiguration($caching, $handler, $lifetime) {
        //Ensure that no cache instances exist
        JFactory::$cache = null;

        //Set the default cache configuration
        JFactory::getConfig()->set('caching', $caching);
        JFactory::getConfig()->set('cache_handler', $handler);
        JFactory::getConfig()->set('cachetime', $lifetime);
    }
    
    private function getSimpleCustomRouterCacheFromJoomlaRouter() {
        $router = JFactory::getApplication()->getRouter();
        $routerRules = $this->getPrivate($router, '_rules');
        
        $simpleCustomRouter = $routerRules['build'][0][0];
        
        return $this->getPrivate($simpleCustomRouter, 'cache');
    }
}

//When a JSession is created (for example, when creating the JApplication), it
//calls to "session_start()". It fails with 
//"Cannot send session cookie - headers already sent by (output started at /usr/share/pear/PHPUnit/Util/Printer.php:174)"
//As the session is not relevant to test the router cache, a JSession stub with
//an empty "_start()" method (the one that called "session_start()" is used.
class JSessionStub extends JSession {
    protected function _start() {
    }
}

==> pkg_simplecustomrouter-0.2.0.zip: unit-tests/plgSystemSimpleCustomRouterTest.php <==
<?php
/**
 * @copyright  Copyright (C) 2012 Daniel Calviño Sánchez
 * @copyright  Copyright (C) 2014 Daniel Calviño Sánchez
 * @copyright  Copyright (C) 2015 Daniel Calviño Sánchez
 * @license    GNU Affero General Public License version 3 or later; see LICENSE.txt
 */

require_once JPATH_BASE.'/tests/unit/core/case/case.php';

require_once JPATH_BASE.'/plugins/system/simplecustomrouter/simplecustomrouter.php';
require_once JPATH_BASE.'/libraries/joomla/event/dispatcher.php';

//Needed by JApplication
require_once JPATH_BASE.'/libraries/legacy/request/request.php';
require_once JPATH_BASE.'/libraries/joomla/uri/uri.php';

class plgSystemSimpleCustomRouterTest extends TestCase {

    protected function setUp() {
        JFactory::$config = null;

        //Set a JSession stub, as creating a real JSession fails (see the
        //comments on JSessionStub class)
        JFactory::$session = new JSessionStub();

        //Set expected server variables.
        if (version_compare(JVERSION, '3.2', 'ge') && !isset($_SERVER['HTTP_HOST'])) {
            $_SERVER['HTTP_HOST'] = 'localhost';
        }

        //In Joomla 3.2 and later, JFactory::getApplication does no longer
        //honour the configuration given because, in turn,
        //JApplicationCms::getInstance does not pass the given configuration to
        //the constructor of JApplicationSite. Disabling the session is a must,
        //because if it is enabled it will try to access the database, so the
        //'site' instance used by JApplicationCms (and returned when calling
        //JApplicationCms::getInstance('site'), like done inside
        //JFactory::getApplication('site')) is forced to a JApplicationSite with
        //a disabled session.
        //Note that a mocked JApplication can not be used, as getRouter() is a
        //static method, and PHPUnit does not support mocking of static methods.
        if (version_compare(JVERSION, '3.2', 'ge')) {
            $application = new JApplicationSite(null, new JRegistry(array('session' => false)));
            $this->setPrivateStaticArrayItem('JApplicationCms', 'instances', 'site', $application);
        }

        JFactory::$application = null;
        //In Joomla! versions previous to 3.2 it is always the same
        //JApplication, as even if the application attribute in JFactory is
        //reset, the internal instance in JApplication is not forced to a new
        //one in each test like done in JApplicationCms for Joomla! 3.2 and
        //later, so the first one created is the one returned over and over.
        JFactory::getApplication('site', array('session' => false));
        //Different JApplications with the same name have the same router (and,
        //therefore, the same JApplication always has the same router), so the
        //router rules have to be emptied between tests.
        $this->clearRouterRules();
    }

    public function testAttachRulesOnInitialization() {
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();

        //Check that the SimpleCustomRouter was added to the rules of the router
        $router = JFactory::getApplication()->getRouter();
        $routerRules = $this->getPrivate($router, '_rules');

        $this->assertEquals(1, count($routerRules['build']), 'Build rules count');
        $this->assertEquals('SimpleCustomRouter', get_class($routerRules['build'][0][0]), 'Build rule class');
        $this->assertEquals('build', $routerRules['build'][0][1], 'Build rule method');

        $this->assertEquals(1, count($routerRules['parse']), 'Parse rules count');
        $this->assertEquals('SimpleCustomRouter', get_class($routerRules['parse'][0][0]), 'Parse rule class');
        $this->assertEquals('parse', $routerRules['parse'][0][1], 'Parse rule method');

        $this->assertTrue($routerRules['build'][0][0] == $routerRules['parse'][0][0], 'Same object in build and parse rules');
    }

    public function testAttachRulesOnAdminInitialization() {
        //Needed for JURI, which is needed by JAdministrator
        $_SERVER['HTTP_HOST'] = 'localhost';

        if (version_compare(JVERSION, '3.2', 'ge')) {
            $application = new JApplicationAdministrator(null, new JRegistry(array('session' => false)));
            $this->setPrivateStaticArrayItem('JApplicationCms', 'instances', 'administrator', $application);
        }

        JFactory::$application = null;
        JFactory::getApplication('administrator', array('session' => false));
        $this->clearRouterRules();

        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();

        //Check that the SimpleCustomRouter was not added to the rules of the
        //router
        $router = JFactory::getApplication()->getRouter();
        $routerRules = $this->getPrivate($router, '_rules');

        //As the rules were cleared there will be no 'build' or 'parse' elements
        if (version_compare(JVERSION, '3.4', 'ge')) {
            $this->assertEquals(0, count($routerRules['build']), 'Build rules count');
            $this->assertEquals(0, count($routerRules['parse']), 'Parse rules count');
        } else {
            $this->assertEquals(0, count($routerRules), 'Rules count');
        }
    }

    private function getPrivate($object, $property) {
        $reflectionProperty = new ReflectionProperty(get_class($object), $property);
        $reflectionProperty->setAccessible(true);
        return $reflectionProperty->getValue($object);
    }

    private function setPrivate($object, $property, $value) {
        $reflectionProperty = new ReflectionProperty(get_class($object), $property);
        $reflectionProperty->setAccessible(true);
        $reflectionProperty->setValue($object, $value);
    }

    private function setPrivateStaticArrayItem($class, $property, $index, $value) {
        $reflectionProperty = new ReflectionProperty($class, $property);
        $reflectionProperty->setAccessible(true);

        $staticArray = $reflectionProperty->getValue();
        $staticArray[$index] = $value;

        $reflectionProperty->setValue($staticArray);
    }

    private function clearRouterRules() {
        // In Joomla! 3.4.0 and later, the attachBuildRule and attachParseRule
        // methods in JRouter ensure that the array where a rule is going to be
        // added exists before adding the rule. Therefore, it is no longer
        // possible to just set an empty array of rules; an array containing one
        // empty array for each type of rule used throughout the tests is needed
        // instead.
        if (version_compare(JVERSION, '3.4', 'ge')) {
            $emptyRulesArray = array(
                'build' => array(),
                'parse' => array()
            );
            $this->setPrivate(JFactory::getApplication()->getRouter(), '_rules', $emptyRulesArray);

            return;
        }

        $this->setPrivate(JFactory::getApplication()->getRouter(), '_rules', array());
    }

    public function testCreateCacheDefaultEnabled() {
        $this->setDefaultCacheConfiguration(true, 'file', '15');

        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();

        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();

        $this->assertNotNull($simpleCustomRouterCache, 'Cache set');
        $this->assertEquals(JFactory::getCache('simplecustomrouter', ''), $simpleCustomRouterCache, 'Cache same as "simplecustomrouter" cache');
        $this->assertTrue($simpleCustomRouterCache->getCaching(), 'Cache enabled');
        $this->assertEquals('JCacheStorageFile', get_class($simpleCustomRouterCache->_getStorage()), 'Cache storage');
        $this->assertEquals('15', $simpleCustomRouterCache->cache->_options['lifetime'], 'Cache lifetime');
    }

    public function testCreateCacheDefaultEnabledWithoutHandler() {
        $this->setDefaultCacheConfiguration(true, '', '15');

        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();

        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();

        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    public function testCreateCacheDefaultDisabled() {
        $this->setDefaultCacheConfiguration(false, 'file', '15');

        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();

        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();

        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    public function testCreateCacheDefaultEnabledWhenCustomDisabled() {
        $this->setDefaultCacheConfiguration(true, 'file', '15');

        $params = '{"overrideDefaultConfiguration":false, "enableCache":false}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();

        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();

        $this->assertNotNull($simpleCustomRouterCache, 'Cache set');
        $this->assertEquals(JFactory::getCache('simplecustomrouter', ''), $simpleCustomRouterCache, 'Cache same as "simplecustomrouter" cache');
        $this->assertTrue($simpleCustomRouterCache->getCaching(), 'Cache enabled');
        $this->assertEquals('JCacheStorageFile', get_class($simpleCustomRouterCache->_getStorage()), 'Cache storage');
        $this->assertEquals('15', $simpleCustomRouterCache->cache->_options['lifetime'], 'Cache lifetime');
    }

    public function testCreateCacheDefaultEnabledWhenCustomDisabledWithoutHandler() {
        $this->setDefaultCacheConfiguration(true, '', '15');

        $params = '{"overrideDefaultConfiguration":false, "enableCache":false}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();

        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();

        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    public function testCreateCacheDefaultDisabledWhenCustomEnabled() {
        $this->setDefaultCacheConfiguration(false, 'file', '15');

        $params = '{"overrideDefaultConfiguration":false, "enableCache":true}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();

        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();

        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    public function testCreateCacheCustomEnabledWhenDefaultDisabled() {
        $this->setDefaultCacheConfiguration(false, 'file', '15');

        $params = '{"overrideDefaultConfiguration":true, "enableCache":true}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();

        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();

        $this->assertNotNull($simpleCustomRouterCache, 'Cache set');
        $this->assertEquals(JFactory::getCache('simplecustomrouter', ''), $simpleCustomRouterCache, 'Cache same as "simplecustomrouter" cache');
        $this->assertTrue($simpleCustomRouterCache->getCaching(), 'Cache enabled');
        $this->assertEquals('JCacheStorageFile', get_class($simpleCustomRouterCache->_getStorage()), 'Cache storage');
        $this->assertEquals('15', $simpleCustomRouterCache->cache->_options['lifetime'], 'Cache lifetime');
    }

    public function testCreateCacheCustomEnabledWhenDefaultDisabledWithoutHandler() {
        $this->setDefaultCacheConfiguration(false, '', '15');

        $params = '{"overrideDefaultConfiguration":true, "enableCache":true}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();

        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();

        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    public function testCreateCacheCustomDisabledWhenDefaultEnabled() {
        $this->setDefaultCacheConfiguration(true, 'file', '15');

        $params = '{"overrideDefaultConfiguration":true, "enableCache":false}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();

        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();

        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    private function setDefaultCacheConfiguration($caching, $handler, $lifetime) {
        //Ensure that no cache instances exist
        JFactory::$cache = null;

        //Set the default cache configuration
        JFactory::getConfig()->set('caching', $caching);
        JFactory::getConfig()->set('cache_handler', $handler);
        JFactory::getConfig()->set('cachetime', $lifetime);
    }

    private function getSimpleCustomRouterCacheFromJoomlaRouter() {
        $router = JFactory::getApplication()->getRouter();
        $routerRules = $this->getPrivate($router, '_rules');

        $simpleCustomRouter = $routerRules['build'][0][0];

        return $this->getPrivate($simpleCustomRouter, 'cache');
    }
}

//When a JSession is created (for example, when creating the JApplication), it
//calls to "session_start()". It fails with
//"Cannot send session cookie - headers already sent by (output started at /usr/share/pear/PHPUnit/Util/Printer.php:174)"
//As the session is not relevant to test the router cache, a JSession stub with
//an empty "_start()" method (the one that called "session_start()" is used.
class JSessionStub extends JSession {
    protected function _start() {
    }
}
